@@ -1,3 +1,4 @@
+from django.http.response import HttpResponseNotFound
 from joplin_vieweb.edit_session import EditSession
 from django.shortcuts import render
 from django.http import HttpResponse
@@ -5,14 +6,14 @@ from django.contrib.auth.decorators import login_required
 from django.conf import settings
 from .joplin import Joplin, ReprJsonEncoder
 import logging
-import markdown
 import json
 from bs4 import BeautifulSoup 
 from pathlib import Path
 import mimetypes
-from .utils import mimetype_to_icon, sync_enable, joplin_sync, markdown_public_ressource
+from .utils import mimetype_to_icon, sync_enable, joplin_sync, markdown_public_ressource, md_to_html
 import threading
 from .edit_session import EditSession
+from .lasts_notes import LastsNotes
 import glob
 
 def conditional_decorator(dec, condition):
@@ -37,20 +38,72 @@ def notebooks(request):
 @conditional_decorator(login_required, settings.JOPLIN_LOGIN_REQUIRED)
 def notes(request, notebook_id):
     joplin = Joplin()
-    notes_metadata = joplin.get_notes_metadata(notebook_id)
-    return render(request, 'joplinvieweb/notes_list.html', {"notes_metadata": notes_metadata})
+    if request.method == "GET": # list the notes of this notebook
+        notes_metadata = joplin.get_notes_metadata(notebook_id)
+        return render(request, 'joplinvieweb/notes_list.html', {"notes_metadata": notes_metadata})
+    if request.method == "POST": # create a notebook
+        data = json.loads(request.body)
+        title = data["title"]
+        parent_id = data["parent_id"]
+        if parent_id == "0":
+            parent_id = ""
+        new_notebook_id = joplin.create_notebook(parent_id, title)
+        return HttpResponse(new_notebook_id)
+
+@conditional_decorator(login_required, settings.JOPLIN_LOGIN_REQUIRED)
+def notebook_delete(request, notebook_id):
+    if request.method == "POST":  # delete the notebook
+        joplin = Joplin()
+        if notebook_id:
+            # first get all the notes of that notebook (recursively, pffff ;-) ) to remove them from last notes:
+            notes_metadata = joplin.get_notes_metadata_recursive(notebook_id)
+            LastsNotes.delete_notes([one_note.id for one_note in notes_metadata])
+            joplin.delete_notebook(notebook_id)
+        return HttpResponse("")
+    return HttpResponseNotFound("")
+
+@conditional_decorator(login_required, settings.JOPLIN_LOGIN_REQUIRED)
+def notebook_rename(request, notebook_id):
+    if request.method == "POST":  # rename the notebook
+        data = json.loads(request.body)
+        title = data["title"]
+        joplin = Joplin()
+        joplin.rename_notebook(notebook_id, title) 
+        return HttpResponse("")
+    return HttpResponseNotFound("")
+
+
+
     
 @conditional_decorator(login_required, settings.JOPLIN_LOGIN_REQUIRED)
 def note(request, note_id, format="html"):
     return HttpResponse(note_body_name(note_id, format)[0])
+    
+@conditional_decorator(login_required, settings.JOPLIN_LOGIN_REQUIRED)
+def note_notebook(request, note_id):
+    return HttpResponse(Joplin().get_note_notebook(note_id))
 
 @conditional_decorator(login_required, settings.JOPLIN_LOGIN_REQUIRED)
 def delete_note(request, note_id):
     joplin = Joplin()
     joplin.delete_note(note_id)
+    LastsNotes.delete_note(note_id)
     return HttpResponse("")
 
 
+@conditional_decorator(login_required, settings.JOPLIN_LOGIN_REQUIRED)
+def render_markdown(request):
+    try:
+        if request.method == "POST":
+            md = json.loads(request.body)
+            md = md["markdown"]
+            html = md_to_html(md, True)
+            return HttpResponse(html)
+    except:
+        pass
+    return HttpResponseNotFound('')
+
+
 def note_body_name(note_id, format, public=False):
     note_body, note_name = Joplin().get_note_body_name(note_id)
     
@@ -61,7 +114,7 @@ def note_body_name(note_id, format, public=False):
         return (note_body, note_name)
     
     note_body = '[TOC]\n\n' + note_body
-    html = markdown.markdown(note_body, extensions=['fenced_code', 'codehilite', 'toc'])
+    html = md_to_html(note_body, False)
     
     # Finally we set an attachment image to the attachments.
     # We search for <a href="/joplin/joplin_ressources"> or <a href=":/">
@@ -75,13 +128,22 @@ def note_body_name(note_id, format, public=False):
             link.insert(0, img)
             link['class'] = link.get('class', []) + ['attachment_link']
         link['target'] = '_blank'
-    html = str(soup)
 
+    toc_item = soup.find('div', {"class": "toc"})
+    if toc_item:
+        for one_link in toc_item.findAll('a'):
+            current_href = str(one_link['href'])
+            new_link = "javascript:scroll_to('" + current_href + "');"
+            one_link['href'] = new_link
+            one_link['target'] = ""
+    html = str(soup)
 
     # Transform [ ] and [x] to checkboxes.
     html = html.replace("<li>[ ] ", '<li><input type="checkbox">');
     html = html.replace("<li>[x] ", '<li><input type="checkbox" checked>');
 
+    LastsNotes.set_last(note_id, note_name)
+
     return (html, note_name)
  
 def public_note(request, note_id):
@@ -105,6 +167,18 @@ def note_checkboxes(request, note_id):
     Joplin().update_note_checkboxes(note_id, cb)
     return HttpResponse("")
     
+@conditional_decorator(login_required, settings.JOPLIN_LOGIN_REQUIRED)
+def pin_note(request, note_id):
+    if request.method == "POST":
+        LastsNotes.pin_note(note_id, True)
+        return HttpResponse("")
+
+@conditional_decorator(login_required, settings.JOPLIN_LOGIN_REQUIRED)
+def unpin_note(request, note_id):
+    if request.method == "POST":
+        LastsNotes.pin_note(note_id, False)
+        return HttpResponse("")
+
 @conditional_decorator(login_required, settings.JOPLIN_LOGIN_REQUIRED)
 def note_tags(request, note_id):
     joplin = Joplin()
@@ -118,6 +192,7 @@ def note_tags(request, note_id):
         joplin.update_note_tags(note_id, tags)
         return HttpResponse("")
 
+
     
     
 
@@ -160,7 +235,7 @@ def joplin_public_ressource(request, ressource_path):
         else:
             response = HttpResponse(content=ressource_file, headers=headers)
     except IOError:
-        response = HttpResponseNotFound()
+        response = HttpResponseNotFound("")
 
     return response
     
@@ -232,7 +307,7 @@ def edit_session_ressource(request, session_id, file):
         else:
             response = HttpResponse(content=ressource_file)
     except IOError:
-        response = HttpResponseNotFound()
+        response = HttpResponseNotFound("")
 
     return response
 
@@ -261,3 +336,7 @@ def edit_session_create_note(request, session_id, notebook_id):
         note_id = joplin.create_note(notebook_id, title, md)
 
     return HttpResponse(note_id)
+
+@conditional_decorator(login_required, settings.JOPLIN_LOGIN_REQUIRED)  
+def get_lasts_notes(request):
+    return HttpResponse(LastsNotes.read_lasts_notes())
